// SPDX-License-Identifier: AGPL-3.0-only
// Copyright © 2020 Bubble, Inc. All Rights Reserved.

import Foundation

protocol Cancellable {
    func cancel()
}

class CancellableContainer: Cancellable {
    var cancellable: Cancellable {
        didSet {
            oldValue.cancel()
            if let container = cancellable as? CancellableContainer {
                cancellable = container.cancellable
            }
        }
    }
    
    init(cancellable: Cancellable) {
        self.cancellable = cancellable
    }
    
    func cancel() {
        cancellable.cancel()
    }
}

extension URLSessionDataTask: Cancellable {
    
}
