// SPDX-License-Identifier: AGPL-3.0-only
// Copyright © 2020 Bubble, Inc. All Rights Reserved.

import Foundation

func wg_log(error: Error, message: String = "") {
    let message = message.isEmpty ? "" : (message + " ")
    wg_log(.error, message: "\(message)\(error.fixedLocalizedDescription)")
}

extension Logger {
    func createLogsFile(completion: @escaping (URL?) -> Void) {
        DispatchQueue(label: "createLogs").async {
            let dateFormatter = ISO8601DateFormatter()
            dateFormatter.formatOptions = [.withFullDate, .withTime, .withTimeZone] // Avoid ':' in the filename
            let timeStampString = dateFormatter.string(from: Date())
            let destinationURL = URL(fileURLWithPath: NSTemporaryDirectory())
                .appendingPathComponent("bubble-log-\(timeStampString).txt")
            try? FileManager.default.removeItem(at: destinationURL)
            var result: URL?
            if self.writeLog(to: destinationURL.path) == true {
                result = destinationURL
            }
            DispatchQueue.main.async {
                completion(result)
            }
        }
    }
}
