// SPDX-License-Identifier: MIT
// Copyright © 2020 Bubble, Inc. All Rights Reserved.

import Foundation

func tryOSStatus(_ status: OSStatus, description: String) throws {
    guard status == noErr else {
        throw NSError(
            description: "\(description): \(SecCopyErrorMessageString(status, nil) as String? ?? "")",
            code: .init(status)
        )
    }
}

extension NSError {
    convenience init(description: String, code: Int = 0) {
        self.init(domain: "Bubble", code: code, userInfo: [NSLocalizedDescriptionKey: description])
    }
}

extension Error {
    var fixedLocalizedDescription: String {
        switch self {
        case let error as WireGuardAppError:
            return error.localizedDescription
            
        default:
            return localizedDescription
        }
    }
}

extension WireGuardAppError {
    var localizedDescription: String {
        "\(alertText.title)\n\(alertText.message)"
    }
}
