// SPDX-License-Identifier: AGPL-3.0-only
// Copyright © 2020 Bubble, Inc. All Rights Reserved.

import Foundation
import SwiftyUserDefaults

struct Credentials: Codable, DefaultsSerializable, CustomStringConvertible {
    let username: String
    let password: String
            
    init?(username: String?,
          password: String?)
    {
        guard ([username, password].first { $0?.isEmpty != false}) == nil else { return nil }
        self.username = username ?? ""
        self.password = password ?? ""
    }
    
    var description: String {
        #if DEBUG
        return "\(username) \(password)"
        #else
        var usernameComponents = username.components(separatedBy: "@")
        usernameComponents[0] = "\(usernameComponents[0].first ?? " ")****\(usernameComponents[0].last ?? " ")"
        return "\(usernameComponents.joined(separator: "@")) \(String(repeating: "*", count: password.count))"
        #endif
    }
    
    static func loadSaved() -> Self? {
        var credentials: Credentials?
        if let path = Bundle.main.path(forResource: "TestCredentials/credentials", ofType: "json"),
            let jsonData = try? Data(contentsOf: URL(fileURLWithPath: path)),
            let testCredentials = try? JSONDecoder().decode(Credentials.self, from: jsonData)
        {
            credentials = testCredentials
        }
        return credentials
    }
}
