// SPDX-License-Identifier: AGPL-3.0-only
// Copyright © 2020 Bubble, Inc. All Rights Reserved.

import QuartzCore

extension CALayer {
    struct Animation<Value> {
        let keyPath: ReferenceWritableKeyPath<CALayer, Value>
        let value: Value
        let duration: TimeInterval
    }
    
    func animate<Value>(_ animation: Animation<Value>, completionHandler: (() -> Void)? = nil) {
        CATransaction.begin()
        CATransaction.setCompletionBlock(completionHandler)
        defer {
            self[keyPath: animation.keyPath] = animation.value
            CATransaction.commit()
        }
        guard animation.duration > 0 else { return }
        let basicAnimation = CABasicAnimation(
            keyPath: NSExpression(forKeyPath: animation.keyPath).keyPath
        )
        basicAnimation.fromValue = self[keyPath: animation.keyPath]
        basicAnimation.toValue = animation.value
        basicAnimation.duration = animation.duration
        
        add(basicAnimation, forKey: basicAnimation.keyPath)
    }
}
