// SPDX-License-Identifier: AGPL-3.0-only
// Copyright © 2020 Bubble, Inc. All Rights Reserved.

#if os(macOS)
import Cocoa
#endif

class GradientButton: Button {
    var gradientProps: GradientView.Props = .mainButton { didSet { gradientView.props = gradientProps } }
    
    private var disabledOverlay: View = {
        let view = UserInteractionDisablableView()
        view.backgroundColor = R.color.disabledButtonOverlay()
        view.alpha = 0
        view.isUserInteractionEnabled = false
        return view
    }()
    
    override var isEnabled: Bool {
        didSet {
            disabledOverlay.alpha = isEnabled ? 0 : 1
        }
    }
    
    #if os(macOS)
    override func hitTest(_ point: NSPoint) -> NSView? {
        guard alpha > 0.01, !isHidden else { return nil }
        return super.hitTest(point)
    }
    #endif
    
    override func awakeFromNib() {
        super.awakeFromNib()
        gradientView.isUserInteractionEnabled = false
        gradientView.translatesAutoresizingMaskIntoConstraints = false
        gradientView.props = gradientProps
        
        insertAtZeroPosition(gradientView)
        let font = R.font.latoBold(size: 14)
        #if os(iOS)
        clipsToBounds = true
        titleLabel?.font = font
        #elseif os(macOS)
        self.font = font
        #endif
        addSubview(disabledOverlay)
    }
    
    override func layoutSubviews() {
        super.layoutSubviews()
        backingLayer.cornerRadius = bounds.height / 2
        [gradientView,
         disabledOverlay,
            ].forEach { $0.frame = bounds }
    }
    
    private let gradientView = GradientView()
}
