// SPDX-License-Identifier: AGPL-3.0-only
// Copyright © 2020 Bubble, Inc. All Rights Reserved.

import UIKit

class ErrorLabel: UILabel {
    @IBOutlet weak var presentationViewController: UIViewController?
    
    var error: Error? {
        didSet {
            updateText()
        }
    }
    
    override var text: String? {
        get { super.text }
        set {
            emptyErrorText = newValue ?? " "
            updateText()
        }
    }
    
    override var textColor: UIColor! {
        get { super.textColor }
        set {
            emptyErrorTextColor = newValue ?? .clear
            updateText()
        }
    }
    
    required init?(coder: NSCoder) {
        super.init(coder: coder)
        isUserInteractionEnabled = true
        if let text = text, !text.isEmpty {
            emptyErrorText = text
        }
        if let textColor = textColor {
            emptyErrorTextColor = textColor
        }
    }
    
    private var emptyErrorTextColor: UIColor?
    private var emptyErrorText = " "
    
    private func updateText() {
        super.text = error?.fixedLocalizedDescription ?? emptyErrorText
        super.textColor = error != nil ? R.color.error() : emptyErrorTextColor
    }
}
