// SPDX-License-Identifier: AGPL-3.0-only
// Copyright © 2020 Bubble, Inc. All Rights Reserved.

import UIKit

private let iPadMultiplier: CGFloat = (UIScreen.main.bounds.width - 768) / (1024 - 768) * 0.3 + 1.3

protocol iPadStretchable {
    var view: UIView! { get }
    var stretchOniPadConstraints: [NSLayoutConstraint]! { get }
    
    func adaptForiPad(block: (CGFloat) -> Void)
}

extension iPadStretchable {
    func adaptForiPad(block: (CGFloat) -> Void = { _ in }) {
        guard UIDevice.current.userInterfaceIdiom == .pad else { return }
        view.adaptSubviewsFontsForiPad()
        stretchOniPadConstraints?.forEach {
            $0.constant *= iPadMultiplier
        }
        block(iPadMultiplier)
    }
}

extension UIView {
    fileprivate func adaptSubviewsFontsForiPad() {
        var subviews = [self]
        while !subviews.isEmpty {
            let view = subviews.removeFirst()
            switch view {
            case let label as UILabel:
                label.font = label.font.scaledFont(multiplier: iPadMultiplier)
                
            case let button as UIButton:
                button.titleLabel?.font = button.titleLabel?.font.scaledFont(multiplier: iPadMultiplier)
                
            case let textField as UITextField:
                textField.font = textField.font?.scaledFont(multiplier: iPadMultiplier)
                
            case let textView as UITextView:
                textView.font = textView.font?.scaledFont(multiplier: iPadMultiplier)
                
            default:
                subviews += view.subviews
            }
        }
    }
}

extension UIFont {
    func scaledFont(multiplier: CGFloat) -> UIFont {
        UIFont(name: fontName, size: pointSize * multiplier)!
    }
}
