// SPDX-License-Identifier: AGPL-3.0-only
// Copyright © 2020 Bubble, Inc. All Rights Reserved.

import Foundation

class XPCClient {
    var connection: NSXPCConnection?
    private let client = InstallationClientImplementation()
    private var installer: Installer? {
        connection?.remoteObjectProxy as? Installer
    }
    
    private func runIfNeeded() {
        guard connection == nil else { return }
        connection = NSXPCConnection(
            machServiceName: FlexrouterConstants.helperMachLabel,
            options: .privileged
        )
        connection?.exportedInterface = NSXPCInterface(with: InstallationClient.self)
        connection?.exportedObject = client
        connection?.remoteObjectInterface = NSXPCInterface(with: Installer.self)
        
        connection?.invalidationHandler = connectionInvalidationHandler
        connection?.interruptionHandler = connectionInterruptionHandler
        
        connection?.resume()
        
    }
    
    func install(
        successHandler: (() -> Void)?,
        unexpectedHandler: ((String) -> Void)?
    ) {
        runIfNeeded()
        client.successHandler = successHandler
        client.unexpectedHandler = unexpectedHandler
        let flexrouterPassword = BubbleKeychain.shared.flexrouterPassword ?? {
            let chars = "abcdefghjklmnpqrstuvwxyz12345789"
            let password =  String((0..<10).map{ _ in chars.randomElement()! })
            BubbleKeychain.shared.flexrouterPassword = password
            return password
        }()
        installer?.install(
            flexrouterPassword: flexrouterPassword
        )
    }
    
    private func connectionInterruptionHandler() {
        wg_log(.error, "[XPC] \(type(of: self)): connection has been interrupted")
        connection = nil
    }
    
    private func connectionInvalidationHandler() {
        wg_log(.error, "[XPC] \(type(of: self)): connection has been invalidated")
        connection = nil
    }
}

private class InstallationClientImplementation: NSObject, InstallationClient {
    var successHandler: (() -> Void)?
    var unexpectedHandler: ((String) -> Void)?
    
    func finishedSuccessfully() {
        successHandler?()
    }
    
    func finishedWithUnexpectedOutput(_ output: String) {
        unexpectedHandler?(output)
    }
}
