// SPDX-License-Identifier: AGPL-3.0-only
// Copyright © 2020 Bubble, Inc. All Rights Reserved.

import Cocoa

class FlexrouterPasswordWindowController: NSWindowController {
    enum State {
        case normal
        case registering
        case incorrectPassword
    }
    
    @IBOutlet weak var passwordTextField: LoginTextField!
    @IBOutlet private var registerButton: NSButton!

    var error: Error? {
        didSet {
            passwordTextField.isError = error != nil
        }
    }
    var state: State = .normal {
        didSet {
            guard state != oldValue else { return }
            var isError = false
            var enabled = true
            switch state {
            case .normal: break
                
            case .registering:
                enabled = false
                
            case .incorrectPassword:
                isError = true
            }
            passwordTextField.isError = isError
            [registerButton,
            passwordTextField,
            ].forEach {
                $0.isEnabled = enabled
            }
            
        }
    }
    var registerHandler: ((String) -> Void)?
    
    override func windowDidLoad() {
        super.windowDidLoad()
        passwordTextField.delegate = self
    }
    
    @IBAction func registerPress(_ sender: NSButton) {
        registerHandler?(passwordTextField.text)
    }
    
    @IBAction func skipPress(_ sender: Any) {
        window?.sheetParent?.endSheet(window!, returnCode: .cancel)
    }
}

extension FlexrouterPasswordWindowController: NSTextFieldDelegate {
    func controlTextDidBeginEditing(_ obj: Notification) {
        error = nil
    }
    
    func controlTextDidChange(_ obj: Notification) {
        registerButton.isEnabled = !passwordTextField.text.isEmpty
    }
}
