// SPDX-License-Identifier: AGPL-3.0-only
// Copyright © 2020 Bubble, Inc. All Rights Reserved.

import Cocoa

extension NSTextField {
    enum SelectionPosition: CaseIterable {
        case start
        case end
    }
    
    var selectedRange: NSRange? {
        get { currentEditor()?.selectedRange }
        set {
            guard let newValue = newValue else { return }
            currentEditor()?.selectedRange = newValue
        }
    }
    
    var selectionPosition: SelectionPosition? {
        SelectionPosition.allCases
            .first { range(for: $0) == selectedRange }
    }
    
    @discardableResult func becomeFirstResponder(selectionPosition: SelectionPosition) -> Bool {
        guard becomeFirstResponder() else { return false }
        selectedRange = range(for: selectionPosition)
        return true
    }
    
    private func range(for selectionPosition: SelectionPosition) -> NSRange {
        switch selectionPosition {
        case .start:
            return .init(location: 0, length: 0)
        case .end:
            return .init(location: text.count, length: 0)
        }
    }
    
    var selectedText: String? {
        guard let selectedRange = selectedRange, selectedRange.length > 0 else { return nil }
        return String(text[selectedRange.lowerBound...selectedRange.upperBound])
    }
 }
