// SPDX-License-Identifier: AGPL-3.0-only
// Copyright © 2020 Bubble, Inc. All Rights Reserved.

import Cocoa

class StatusItemController {
    let statusItem = NSStatusBar.system.statusItem(withLength: NSStatusItem.squareLength)

    private let statusBarImageWhenActive = NSImage(named: "StatusBarIcon")!
    private let statusBarImageWhenInactive = NSImage(named: "StatusBarIconDimmed")!
    
    private var statusKVO: AnyObject?

    init(connectionManager: ConnectionManager) {
        statusKVO = connectionManager.addStatusChangedHandler(statusHandler: .init({ [weak self] status in
            self?.statusChanged(status)
        }))
    }

    private func statusChanged(_ status: ConnectionManager.Status) {
        switch status {
        case .active:
            statusItem.button?.image = statusBarImageWhenActive
        default:
            statusItem.button?.image = statusBarImageWhenInactive
        }
    }
}
