// SPDX-License-Identifier: AGPL-3.0-only
// Copyright © 2020 Bubble, Inc. All Rights Reserved.

import Cocoa

class StatusMenu: NSMenu {
    var statusMenuItem: NSMenuItem!
    var actionMenuItem: NSMenuItem!

    var statusKVO: AnyObject?

    let connectionManager: ConnectionManager
    
    init(connectionManager: ConnectionManager) {
        self.connectionManager = connectionManager
        super.init(title: "Bubble")
        
        statusMenuItem = addItem(title: tr(format: "macStatus (%@)", tr("tunnelStatusInactive")))
        statusMenuItem.isEnabled = false
        
        actionMenuItem = addItem(title: "", target: self, action: #selector(actionClicked))
        actionMenuItem.isHidden = true
        addItem(NSMenuItem.separator())

        addItem(title: "Log out", target: NSApp.delegate, action: #selector(AppDelegate.logOut))
        addItem(title: "About", target: NSApp.delegate, action: #selector(AppDelegate.aboutClicked))
        addItem(title: "Quit", target: NSApp.delegate, action: #selector(AppDelegate.quit))
        
        statusKVO = connectionManager.addStatusChangedHandler(statusHandler: .init({ [weak self] status in
            self?.statusChanged(status)
        }))
    }

    required init(coder decoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    private func statusChanged(_ status: ConnectionManager.Status) {
        statusMenuItem.title = tr(format: "macStatus (%@)", status.localizedStatusDescription)
        actionMenuItem.isHidden = status == .initial
        actionMenuItem.title = status.localizedToggleStatusActionText
    }

    @objc func actionClicked() {
        connectionManager.toggleStatus()
    }
    
    @discardableResult private func addItem(title string: String,
                                            target: AnyObject? = nil,
                                            action selector: Selector? = nil,
                                            keyEquivalent charCode: String = "") -> NSMenuItem {
        let aboutItem = NSMenuItem(title: string, action: selector, keyEquivalent: charCode)
        aboutItem.target = target
        addItem(aboutItem)
        return aboutItem
    }
}
