#!/bin/bash

if [[ $(whoami) == "root" ]] ; then
    su - jenkins -c $0
    exit $?
fi

TUNNEL_TARGET="jenkins.bubblev.org"

# Can we get to the public URL?
PUBLIC_URL="http://xipper.bubblev.org/"
TUNNEL_STAT=$(curl -s -o /dev/null -w "%{http_code}" ${PUBLIC_URL})

TUNNEL_OK=1
if [[ -z "${TUNNEL_STAT}" ]] ; then
    TUNNEL_OK=0
else
    TUNNEL_STAT_TYPE=$(echo ${TUNNEL_STAT} | head -c 1)
    if [[ ${TUNNEL_STAT_TYPE} -ne 2 ]] ; then
	TUNNEL_OK=0
    fi
fi

LOG=${HOME}/xipper-tunnel.log
if [[ ${TUNNEL_OK} -eq 1 ]] ; then
    echo "$(date): tunnel already open and working" | tee -a ${LOG}
    exit 0
else
    ps -ef | grep ssh | grep -v grep | grep "${TUNNEL_TARGET}" | awk '{print $2}' | \
	while read pid ; do
	    echo "$(date): killing existing non-functional tunnel with pid ${pid}" | tee -a ${LOG}
	    kill ${pid}
	done
fi

echo "$(date): establishing tunnel" | tee -a ${LOG}
set -o pipefail
PORTS="-R 1080:127.0.0.1:80 -R 2222:127.0.0.1:22"
/usr/local/bin/autossh -M 0 -f -T -N -o "ServerAliveInterval 30" -o "ServerAliveCountMax 3" ${PORTS} "${TUNNEL_TARGET}" 2>&1 | tee -a ${LOG} && \
    echo "$(date): tunnel established" | tee -a ${LOG}
